<?php

namespace CRPlugins\Oca\Orders;

use CRPlugins\Oca\Helper\Helper;

defined( 'ABSPATH' ) || exit;

class Metabox {

	public function __construct() {
		add_action( 'add_meta_boxes', array( $this, 'create' ) );
	}

	public function create(): void {
		$order_types = wc_get_order_types( 'order-meta-boxes' );
		foreach ( $order_types as $order_type ) {
			add_meta_box(
				'oca_metabox',
				'OCA',
				array( $this, 'content' ),
				$order_type,
				'side',
				'default'
			);
		}
	}

	public function content( $post, $metabox ): void {
		$order = wc_get_order( $post->ID );
		if ( ! $order ) {
			return;
		}
		wp_enqueue_style( 'wc-oca-general-css' );
		wp_enqueue_style( 'wc-oca-orders-css' );

		$shipping_method = Helper::get_order_shipping_method( $order );
		if ( ! $shipping_method ) {
			return;
		}
		wp_enqueue_script( 'wc-oca-orders-js' );
		wp_localize_script(
			'wc-oca-orders-js',
			'wc_oca_settings',
			array(
				'ajax_url'            => admin_url( 'admin-ajax.php' ),
				'ajax_nonce'          => wp_create_nonce( 'wc-oca' ),
				'order_id'            => $order->get_id(),
				'oca_number'          => $shipping_method->get_meta( 'oca_number' ),
				'oca_tracking_number' => $shipping_method->get_meta( 'tracking_number' ),
			)
		);
		wp_localize_script(
			'wc-oca-orders-js',
			'wc_oca_translation_texts',
			array(
				'generic_error_try_again' => esc_html__( 'There was an error, please try again', 'wc-oca' ),
				'invalid_package_number'  => esc_html__( 'Input a valid package number', 'wc-oca' ),
				'packages_qty_updated'    => esc_html__( 'Packages quantity updated', 'wc-oca' ),
				'loading'                 => esc_html__( 'Loading...', 'wc-oca' ),
				'track_order'             => esc_html__( 'Track order', 'wc-oca' ),
				'view_shipping_label_pdf' => esc_html__( 'View shipping label', 'wc-oca' ),
				'view_shipping_label_zpl' => esc_html__( 'Download shipping label ZPL', 'wc-oca' ),
				'process_order_now'       => esc_html__( 'Process order now', 'wc-oca' ),
				'cancel_order'            => esc_html__( 'Cancel shipment', 'wc-oca' ),
				'new_quote_detected'      => esc_html__( 'A new shipping cost was detected for the new data of the order (based on shipping address and items). Do you want to proceed? This additional cost will not be charged automatically to the customer, make sure to cover it by yourself. Old price: {{old_price}} | New price: {{new_price}}', 'wc-oca' ),
			)
		);
		if ( Helper::shipping_method_is_oca( $shipping_method ) ) {
			$oca_packages = $shipping_method->get_meta( 'oca_packages' );

			$canceled = $shipping_method->get_meta( 'oca_canceled' );
			if ( $canceled ) {
				$this->show_order_canceled( $oca_packages );
				return;
			}

			$tracking_number = $shipping_method->get_meta( 'tracking_number' );
			if ( ! $tracking_number ) {
				$this->show_order_not_processed_yet( $order );
				$this->show_packages_number( $oca_packages );
				return;
			}

			$this->show_order_processed( $tracking_number );

		} else {
			echo esc_html__( 'This order is not using OCA as shipping method', 'wc-oca' );
		}
	}


	protected function show_order_processed( string $tracking_number ): void {
		printf( __( 'The order has been processed, tracking number: <strong>%s</strong>', 'wc-oca' ), esc_html( $tracking_number ) );
		echo '<a class="oca-button block" id="oca-view-shipping-label-pdf-a4">' . esc_html__( 'View shipping label PDF - A4', 'wc-oca' ) . '</a>';
		echo '<a class="oca-button block" id="oca-view-shipping-label-pdf-10x15">' . esc_html__( 'View shipping label PDF - 10x15', 'wc-oca' ) . '</a>';
		echo '<a class="oca-button block" id="oca-download-shipping-label-pdf-a4">' . esc_html__( 'Download shipping label PDF - A4', 'wc-oca' ) . '</a>';
		echo '<a class="oca-button block" id="oca-download-shipping-label-pdf-10x15">' . esc_html__( 'Download shipping label PDF - 10x15', 'wc-oca' ) . '</a>';
		echo '<a class="oca-button block" id="oca-download-shipping-label-zpl">' . esc_html__( 'Download shipping label ZPL', 'wc-oca' ) . '</a>';
		echo '<a class="oca-button block" id="oca-view-tracking">' . esc_html__( 'Track order', 'wc-oca' ) . '</a>';
		echo '<a class="oca-red-button block" id="oca-cancel-order">' . esc_html__( 'Cancel order', 'wc-oca' ) . '</a>';
	}

	protected function show_order_canceled( string $packages_number ): void {
		echo '<p>';
		echo esc_html__( 'The shipment has been canceled', 'wc-oca' );

		$this->show_packages_number( $packages_number );
		echo '<br>';

		// Process now button.
		$this->show_process_now_button();
		echo '</p>';
	}

	protected function show_order_not_processed_yet( \WC_Order $order ): void {
		echo '<p>';
		echo esc_html__( 'The order is not processed yet', 'wc-oca' );
		echo '</p>';

		// Status config.
		$config_status = Helper::get_option( 'status_processing' );
		if ( ! empty( $config_status ) ) {
			$config_status = str_replace( 'wc-', '', $config_status );
			if ( ! $order->has_status( $config_status ) && '0' !== $config_status ) {
				$statuses = wc_get_order_statuses();
				// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				printf( __( 'The order will be processed when its status is <strong>%s</strong>', 'wc-oca' ), $statuses[ 'wc-' . $config_status ] );
			}
		}

		$this->show_process_now_button();
	}

	protected function show_process_now_button(): void {
		// Process now button.
		echo '<a class="oca-button block" target="_blank" id="oca_process_order">' . esc_html__( 'Process order now', 'wc-oca' ) . '</a>';
	}

	protected function show_packages_number( string $packages_number ): void {
		echo '<div class="oca-packages-div-wrapper">';
		echo esc_html__( 'Number of packages', 'wc-oca' );
		echo '<div class="oca-packages-wrapper">';
		echo '<input type="number" step="1" id="oca_packages_qty" value="' . esc_attr( $packages_number ) . '" />';
		echo '<a href="#" class="oca-button save-button edit-packages-button">' . esc_html__( 'Edit', 'wc-oca' ) . '</a>';
		echo '</div>';
		echo '</div>';
	}
}
