<?php

namespace CRPlugins\Oca\Sdk;

use CRPlugins\Oca\Api\OcaApi;
use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\ValueObjects\Seller;
use CURLFILE;
use DateTime;
use Exception;
use WC_Order;

class OcaSdk {

	/**
	 * @var OcaApi
	 */
	private $api;

	/**
	 * @var array
	 */
	private $settings;

	public function __construct() {
		$this->api      = new OcaApi( Helper::get_option( 'apikey' ) );
		$this->settings = array(
			'username'      => null,
			'password'      => null,
			'accountNumber' => null,
			'cuit'          => null,
			'shippingCode'  => null,
		);
	}

	public function set_data( Seller $seller ) {
		$this->settings = array(
			'username'      => $seller->get_username(),
			'password'      => $seller->get_password(),
			'accountNumber' => $seller->get_account_number(),
			'cuit'          => $seller->get_cuit(),
		);
	}

	/**
	 * Sets shipping code
	 *
	 * @param string $shipping_code
	 * @return void
	 */
	public function set_shipping_code( string $shipping_code ) {
		$this->settings['shippingCode'] = $shipping_code;
	}

	/**
	 * Sets shipping codes
	 *
	 * @param array $shipping_codes OCA's shipping codes
	 * @return void
	 */
	public function set_shipping_codes( array $shipping_codes ) {
		$this->settings['shippingCodes'] = $shipping_codes;
	}

	/**
	 * Gets API key license status
	 *
	 * @return array license status
	 */
	public function get_license_status(): array {
		try {
			$res = $this->api->get( '/store' );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	protected function handle_response( array $response, string $function_name ): array {
		if ( Helper::is_debug_enabled() ) {
			Helper::log_debug( sprintf( __( '%1$s - Data sent to Oca: %2$s', 'wc-oca' ), $function_name, print_r( $response['request'], true ) ) );
			Helper::log_debug( sprintf( __( '%1$s - Data received from Oca: %2$s', 'wc-oca' ), $function_name, print_r( $response['response'], true ) ) );
		} elseif ( ! Helper::is_debug_enabled() && 'process_order' === $function_name ) {
			Helper::log_debug( sprintf( __( '%1$s - Data sent to Oca: %2$s', 'wc-oca' ), $function_name, print_r( $response['request'], true ) ) );
			Helper::log_debug( sprintf( __( '%1$s - Data received from Oca: %2$s', 'wc-oca' ), $function_name, print_r( $response['response'], true ) ) );
		}

		if ( empty( $response['response'] ) ) {
			Helper::log_warning( $function_name . ': ' . __( 'No response from OCA server', 'wc-oca' ) );
			return array( 'error' => __( 'No response from OCA server', 'wc-oca' ) );
		}

		if ( ! empty( $response['response']['error'] ) ) {
			Helper::log_error( $function_name . ': ' . $response['response']['error'] );
			Helper::log_error( 'Data sent: ' . print_r( $response['request'], true ) );
		}

		return $response['response'];
	}

	/**
	 * Gets OCA's user data
	 *
	 * @return array user data
	 */
	public function get_user_data(): array {
		if ( ! $this->has_data() ) {
			throw new Exception( 'OCA Sdk must be initialized before being used' );
		}
		$data_to_send = array(
			'settings' => $this->settings,
		);
		try {
			$res = $this->api->post( '/oca/user', $data_to_send );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Returns true if the sdk has necessary data to operate
	 *
	 * @param boolean $strict strict mode
	 * @return boolean true if the sdk has necessary data to operate. False otherwise
	 */
	public function has_data( bool $strict = false ): bool {
		if ( ! $strict ) {
			return ( ! empty( $this->settings['username'] ) && ! empty( $this->settings['password'] ) );
		} else {
			return ( $this->has_data() && ! empty( $this->settings['accountNumber'] ) && ! empty( $this->settings['cuit'] ) && ( ! empty( $this->settings['shippingCode'] ) || ! empty( $this->settings['shippingCodes'] ) ) );
		}
	}

	/**
	 * Gets a quote for an order
	 *
	 * @param array $from
	 * @param array $to
	 * @param array $items
	 * @return array
	 */
	public function get_price( array $cart, string $from_postcode, string $to_postcode ): array {
		if ( ! $this->has_data( true ) ) {
			throw new Exception( 'OCA Sdk must be initialized before being used' );
		}
		$data_to_send = array(
			'totalWeight' => $cart['weight'],
			'totalVolume' => $cart['volume'],
			'price'       => $cart['price'],
			'qty'         => $cart['quantity'],
			'from'        => array(
				'postcode' => $from_postcode,
			),
			'to'          => array(
				'postcode' => $to_postcode,
			),
			'settings'    => $this->settings,
		);
		try {
			$res = $this->api->post( '/oca/quote', $data_to_send );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	public function process_order( Seller $seller, array $to, array $items, WC_Order $order, array $opts = array() ): array {
		if ( ! $this->has_data( true ) ) {
			throw new Exception( 'OCA Sdk must be initialized before being used' );
		}
		$now            = new DateTime();
		$seller_address = $seller->get_address();
		$data_to_send   = array(
			'amount'       => $order->get_total(),
			'orderId'      => $order->get_id(),
			'forceProcess' => empty( $opts['force_process'] ) ? false : true,
			'from'         => array(
				'street'         => $seller_address->get_street(),
				'streetNumber'   => $seller_address->get_number(),
				'floor'          => $seller_address->get_floor(),
				'apt'            => $seller_address->get_apartment(),
				'postcode'       => $seller_address->get_postcode(),
				'city'           => $seller_address->get_city(),
				'state'          => $seller_address->get_state(),
				'contact'        => $seller_address->get_name(),
				'email'          => $seller_address->get_email(),
				'storeName'      => $seller_address->get_store_name(),
				'observations'   => $seller_address->get_observations(),
				'costBranch'     => $seller_address->get_cost_branch(),
				'timeFrame'      => $seller_address->get_time_frame(),
				'shippingBranch' => $seller_address->get_shipping_branch(),
				'date'           => $now->format( 'Ymd' ),
			),
			'to'           => array(
				'firstName'      => $to['first_name'],
				'lastName'       => $to['last_name'],
				'street'         => $to['street'],
				'streetNumber'   => $to['number'],
				'floor'          => $to['floor'],
				'apt'            => $to['apartment'],
				'city'           => $to['city'],
				'state'          => $to['state'],
				'postcode'       => $to['postcode'],
				'phone'          => $to['phone'],
				'email'          => $to['email'],
				'shippingBranch' => $to['shipping_branch'],
				'observations'   => $to['extra_info'],
			),
			'items'        => array(),
			'settings'     => $this->settings,
		);
		foreach ( $items as $item ) {
			$item_array              = $item->to_array();
			$item_array['qty']       = $item_array['quantity'];
			$data_to_send['items'][] = $item_array;
		}
		try {
			$res = $this->api->post( '/oca/orders', $data_to_send );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Cancels an order in Oca's Api
	 *
	 * @return array
	 */
	public function cancel_order( string $oca_number ): array {
		if ( ! $this->has_data() ) {
			throw new Exception( 'OCA Sdk must be initialized before being used' );
		}
		$data_to_send = array(
			'ocaNumber' => $oca_number,
			'settings'  => $this->settings,
		);
		try {
			$res = $this->api->post( '/oca/orders/cancel', $data_to_send );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Gets the shipping label in pdf for a Oca Shipment
	 *
	 * @param string $oca_number
	 * @return array
	 */
	public function get_shipping_label_pdf_a4( string $oca_number ): array {
		$query_params = http_build_query(
			array(
				'number' => $oca_number,
				'size'   => 'A4',
			)
		);

		try {
			$res = $this->api->get( '/oca/label?' . $query_params );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	public function get_shipping_label_pdf_10x15( string $oca_number ): array {
		$query_params = http_build_query(
			array(
				'number' => $oca_number,
				'size'   => '10x15',
			)
		);

		try {
			$res = $this->api->get( '/oca/label?' . $query_params );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Gets the shipping label in zpl for a Oca Shipment
	 *
	 * @param string $oca_number
	 * @return array
	 */
	public function get_shipping_label_zpl( string $oca_number ): array {
		try {
			$res = $this->api->get( '/oca/labelzpl?number=' . $oca_number );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	public function get_all_shipping_labels( string $oca_number ): array {
		try {
			$res = $this->api->get( '/oca/labels?number=' . $oca_number );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array(
				'pdf-a4'    => '',
				'pdf-10x15' => '',
				'zpl'       => '',
			);
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Gets the tracking status for a Oca Shipment
	 *
	 * @param string $tracking_number
	 * @return array
	 */
	public function get_tracking( string $tracking_number ): array {
		try {
			$res = $this->api->get( '/oca/tracking?number=' . $tracking_number );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Gets the tracking status for a Oca Shipment
	 *
	 * @param string $tracking_number
	 * @return array
	 */
	public function get_shipping_branches_to( string $postcode ): array {
		try {
			$res = $this->api->get( '/oca/branches/to?number=' . $postcode );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	/**
	 * Gets the tracking status for a Oca Shipment
	 *
	 * @param string $tracking_number
	 * @return array
	 */
	public function get_shipping_branches_from( string $postcode ): array {
		try {
			$res = $this->api->get( '/oca/branches/from?number=' . $postcode );
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}

	public function send_report( string $report_uri ): array {
		try {
			$curl = curl_init();
			curl_setopt_array(
				$curl,
				array(
					CURLOPT_URL            => 'https://ocaapi.crplugins.com.ar/api/v1/oca/reports',
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_CUSTOMREQUEST  => 'POST',
					CURLOPT_POSTFIELDS     => array( 'report' => new CURLFILE( $report_uri ) ),
					CURLOPT_HTTPHEADER     => array(
						'origin: ' . get_site_url(),
						'Authorization: ' . Helper::get_option( 'apikey' ),
					),
				)
			);
			$res = curl_exec( $curl );
			curl_close( $curl );
			$res = array(
				'request'  => array( 'report' => new CURLFILE( $report_uri ) ),
				'response' => json_decode( $res, true ),
			);
		} catch ( Exception $e ) {
			Helper::log_error( __FUNCTION__ . ': ' . $e->getMessage() );
			return array();
		}
		return $this->handle_response( $res, __FUNCTION__ );
	}
}
