<?php

namespace CRPlugins\Oca\Settings;

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\Sdk\OcaSdk;
use CRPlugins\Oca\Settings\Sections\IntegrationSection;
use CRPlugins\Oca\Settings\Sections\OcaSection;
use CRPlugins\Oca\Settings\Sections\PluginSection;
use CRPlugins\Oca\Settings\Sections\SellerSection;

defined( 'ABSPATH' ) || exit;

/**
 * A main class that holds all our settings logic
 */
class Main {

	/**
	 * Gets all settings fields from all the settings sections
	 *
	 * @return array
	 */
	public static function get_settings_fields() {
		return array_merge(
			PluginSection::get_fields(),
			OcaSection::get_fields(),
			SellerSection::get_fields(),
			IntegrationSection::get_fields()
		);
	}

	/**
	 * Gets all settings (options registered with their values)
	 *
	 * @return array
	 */
	public static function get_all_settings() {
		$settings = self::get_settings_fields();
		$data     = array();
		foreach ( $settings as $setting ) {
			$data[ $setting['slug'] ] = Helper::get_option( $setting['slug'] );
		}
		return $data;
	}

	/**
	 * Registers the sections and render them
	 *
	 * @return void
	 */
	public static function init_settings() {
		register_setting( 'wc-oca', 'wc-oca_options' );

		$section = new PluginSection();
		$section->add();
		if ( Helper::get_option( 'apikey' ) ) {
			$section = new OcaSection();
			$section->add();
			$section = new SellerSection();
			$section->add();
			$section = new IntegrationSection();
			$section->add();
		}
	}

	/**
	 * Adds our assets into our settings page
	 *
	 * @param string $hook
	 * @return void
	 */
	public static function add_assets_files( string $hook ) {
		if ( 'toplevel_page_wc-oca-admin' === $hook ) {
			wp_enqueue_style( 'wc-oca-general-css' );
			wp_enqueue_style( 'wc-oca-settings-css' );
			wp_enqueue_script( 'wc-oca-settings-js' );
			wp_localize_script(
				'wc-oca-settings-js',
				'wc_oca_settings',
				array(
					'ajax_url'      => admin_url( 'admin-ajax.php' ),
					'ajax_nonce'    => wp_create_nonce( 'wc-oca' ),
					'hasapikey'     => ! empty( Helper::get_option( 'apikey' ) ),
					'shippingCodes' => wp_json_encode( Helper::get_option( 'shippingCodes' ) ),
				)
			);
			wp_localize_script(
				'wc-oca-settings-js',
				'wc_oca_translation_texts',
				array(
					'user_info_pre_check'                => __( 'You must enter a valid username and password before retrieving the your user data', 'wc-oca' ),
					'user_info_success'                  => __( 'User info retrieval success', 'wc-oca' ),
					'user_info_fail'                     => __( 'User info retrieval failed', 'wc-oca' ),
					'user_info_fail_credentials'         => __( 'User could not be retrieved, check your credentials or input your data manually', 'wc-oca' ),
					'generic_error_try_again'            => __( 'There was an error, please try again', 'wc-oca' ),
					'delete_settings_confirmation'       => __( 'Are you sure you want to delete all your oca settings?', 'wc-oca' ),
					'delete_settings_success'            => __( 'Settings were deleted succesfully', 'wc-oca' ),
					'delete_settings_fail'               => __( 'Unable to delete settings, please try again', 'wc-oca' ),
					'report_sent_success'                => __( 'The report has been sent', 'wc-oca' ),
					'report_sent_fail'                   => __( 'The report could not been sent, please try again', 'wc-oca' ),
					'validate_credentials'               => __( 'Validate credentials', 'wc-oca' ),
					'shipping_branch_finder_popup_title' => __( 'OCA - Shipping branch finder', 'wc-oca' ),
				)
			);
		}
	}

	/**
	 * Creates a setting option in the WordPress Sidebar
	 *
	 * @return void
	 */
	public static function create_menu_option() {
		add_menu_page(
			'OCA',
			'OCA',
			'manage_options',
			'wc-oca-admin',
			array( __CLASS__, 'settings_page_content' ),
			Helper::get_assets_folder_url() . '/img/logo.svg'
		);

		add_submenu_page(
			'wc-oca-admin',
			'OCA',
			__( 'Main configuration', 'wc-oca' ),
			'manage_options',
			'wc-oca-admin',
			array( __CLASS__, 'settings_page_content' ),
			10
		);
	}

	/**
	 * Displays the settings pages
	 *
	 * @return void
	 */
	public static function settings_page_content() {

		if ( ! is_admin() || ! current_user_can( 'manage_options' ) ) {
			die( 'what are you doing here?' );
		}

		$nonce = $_REQUEST['_wpnonce'] ?? null;
		if ( ! empty( $_POST ) && $nonce && ! wp_verify_nonce( $nonce, 'wc-oca-settings-options' ) ) {
			die( 'what are you doing here?' );
		}

		// Prepare shipping codes for saving
		if ( ! empty( $_POST['shippingCodes'] ) && ! $_POST['shippingCodes'][1]['id'] ) {
			$shipping_codes         = array_filter(
				$_POST['shippingCodes'],
				function ( $value ) {
					return ! empty( $value['id'] );
				}
			);
			$_POST['shippingCodes'] = array_values( $shipping_codes );
		}

		$errors         = array();
		$settings_saved = FieldsVerifier::save_settings( $_POST, $errors );

		// Refresh api key status
		if ( Helper::get_option( 'apikey' ) && empty( $_POST['apikey'] ) ) {
			$oca_sdk = new OcaSdk();
			$status  = $oca_sdk->get_license_status();
			if ( ! empty( $status['expiration_date'] ) ) {
				Helper::save_option( 'membership', $status['expiration_date'] );
			}
		}

		// Validate Api key
		if ( ! empty( $_POST['apikey'] ) ) {
			$oca_sdk = new OcaSdk();
			$status  = $oca_sdk->get_license_status();
			if ( empty( $status ) || ! empty( $status['error'] ) ) {
				Helper::add_error( __( 'Api key details couldn\'t be retrieved, please try again', 'wc-oca' ), true );
				Helper::delete_option( 'membership' );
				Helper::delete_option( 'apikey' );
			} else {
				Helper::save_option( 'membership', $status['expiration_date'] );
			}
		}

		if ( $settings_saved && Helper::get_option( 'apikey' ) ) {
			Helper::add_success( __( 'Settings saved', 'wc-oca' ) );
			if ( ! empty( $errors ) ) {
				foreach ( $errors as $error ) {
					Helper::add_error( $error );
				}
			}
			wp_enqueue_script( 'wc-oca-reload-js' );
		}

		$logo_url = Helper::get_assets_folder_url() . '/img/logo.png';
		?>
		<div class="oca-form-wrapper wrap">
			<div class="settings-header">
				<img src="<?php echo $logo_url; ?>" class="logo">
			</div>
			<form action="admin.php?page=wc-oca-admin" method="post" class="form-wrapper">
				<?php
				settings_fields( 'wc-oca-settings' );
				do_settings_sections( 'wc-oca-settings' );
				submit_button( __( 'Save settings', 'wc-oca' ) );
				if ( Helper::get_option( 'apikey' ) ) :
					?>
					<a class="oca-red-button" id="oca-delete-settings"><?php echo __( 'Delete options', 'wc-oca' ); ?></a>
					<?php
				endif;
				?>
			</form>
		</div>
		<?php

	}
}
