<?php

namespace CRPlugins\Oca\ShippingMethod;

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\Sdk\OcaSdk;
use CRPlugins\Oca\ShippingLabels\AllLabelsProcessor;
use Exception;
use WC_Order;

class OrderProcessor {

	/**
	 * @var OcaSdk
	 */
	private $oca_sdk;

	/**
	 * @var AllLabelsProcessor
	 */
	private $all_labels_processor;

	public function __construct() {
		add_action( 'woocommerce_checkout_process', array( $this, 'check_data_before_place_order' ) );
		add_action( 'woocommerce_order_status_changed', array( $this, 'handle_order_status' ), 10, 4 );

		$this->oca_sdk              = new OcaSdk();
		$this->all_labels_processor = new AllLabelsProcessor();
	}

	public function check_data_before_place_order(): void {
		$chosen_methods = WC()->session->get( 'chosen_shipping_methods' );
		if ( empty( $chosen_methods[0] ) ) {
			return;
		}

		// if oca to branch, check if branch exists
		$is_oca = Helper::cart_method_is_oca( $chosen_methods[0] );
		if ( $is_oca['branch'] && empty( WC()->session->get( 'wc_oca_selected_shipping_branch' ) ) ) {
			wc_add_notice( __( 'Please select a OCA shipping branch', 'wc-oca' ), 'error' );
		}
	}

	public function cancel_order( WC_Order $order ) : void {
		$shipping_method = Helper::get_order_shipping_method( $order );
		if ( ! Helper::shipping_method_is_oca( $shipping_method ) ) {
			return;
		}

		$oca_number = $shipping_method->get_meta( 'oca_number' );
		if ( empty( $oca_number ) ) {
			Helper::add_error( __( 'Could not cancel the shipment, because the order has not been processed yet', 'wc-oca' ) );
			return;
		}

		$seller = Helper::get_seller_from_settings();
		if ( empty( $seller->get_account_number() ) ) {
			Helper::add_error( __( 'There was an error canceling the shipment with OCA, please try again', 'wc-oca' ) );
			return;
		}

		$this->oca_sdk->set_data( $seller );
		$res = $this->oca_sdk->cancel_order( $oca_number );

		// oca id 100 - Canceled | id 130 - Already canceled
		if ( ! empty( $res ) && empty( $res['error'] ) && ( '100' === $res['IdResult'] || '130' === $res['IdResult'] ) ) {
			$shipping_method->update_meta_data( 'oca_canceled', 'true' );
			$shipping_method->save();

			$this->all_labels_processor->delete_all_labels( $order );

			$order->add_order_note( sprintf( __( 'OCA shipment for order %s has been canceled', 'wc-oca' ), $order->get_id() ) );
			Helper::add_success( sprintf( __( 'OCA shipment for order %s has been canceled', 'wc-oca' ), $order->get_id() ) );

			do_action( 'wc_oca_res_after_cancel', $shipping_method, $order );
		} else {
			if ( ! empty( $res['error'] ) ) {
				Helper::add_error( $res['error'] );
			} elseif ( ! empty( $res['Mensaje'] ) ) {
				Helper::add_error( $res['Mensaje'] );
			} else {
				Helper::add_error( __( 'There was an error canceling the shipment with OCA, please try again', 'wc-oca' ) );
			}
		}
	}

	public function handle_order_status( int $order_id, string $status_from, string $status_to, WC_Order $order ) {
		$shipping_method = Helper::get_order_shipping_method( $order );
		if ( $shipping_method && Helper::shipping_method_is_oca( $shipping_method ) ) {
			$config_status = Helper::get_option( 'status_processing' );
			$config_status = str_replace( 'wc-', '', $config_status );

			if ( $order->has_status( $config_status ) && empty( $shipping_method->get_meta( 'tracking_number' ) ) ) {
				$this->process_order( $order );
			}
		}
	}

	public function process_order( WC_Order $order, array $opts = array() ): void {
		$shipping_method = Helper::get_order_shipping_method( $order );
		if ( ! $shipping_method ) {
			return;
		}

		if ( ! Helper::shipping_method_is_oca( $shipping_method )
			|| ( ! empty( $shipping_method->get_meta( 'tracking_number' ) ) && empty( $opts['force_process'] ) )
			) {
			return;
		}

		$shipping_code = $shipping_method->get_meta( 'shipping_code' );
		if ( empty( $shipping_code['code'] ) ) {
			if ( empty( $shipping_code['id'] ) ) {
				Helper::add_error( __( 'There was an error processing the order with OCA, please try again', 'wc-oca' ) );
				return;
			} else {
				$shipping_code['code'] = $shipping_code['id']; // BW compat, previously 'id' was used, not anymore.
			}
		}

		// From
		$seller = Helper::get_seller_from_settings();
		if ( empty( $seller->get_account_number() ) ) {
			Helper::add_error( __( 'There was an error processing the order with OCA, please try again', 'wc-oca' ) );
			return;
		}

		// To
		$to = Helper::get_order_customer( $order );

		// Items
		try {
			$items = Helper::get_items_from_order( $order );
		} catch ( Exception $e ) {
			if ( Helper::is_debug_enabled() ) {
				Helper::log_warning( __FUNCTION__ . ': ' . $e->getMessage() );
			}
			Helper::add_error( __( 'There was an error processing the order with OCA, check the order items and try again', 'wc-oca' ) );
			return;
		}

		$override_packages = (int) $shipping_method->get_meta( 'oca_packages' );
		if ( $items->totals()['quantity'] !== $override_packages ) {
			// Number of packages overrided, send a custom item
			$default_size = Helper::get_option( 'default_package_size' );
			$default_size = explode( 'x', $default_size );
			$items->set_custom_items_quantity( $override_packages, $default_size );
		}

		// Considerations for shipping to branch office
		if ( 'SaS' === $shipping_code['type'] || 'PaS' === $shipping_code['type'] ) {
			$selected_branch = $shipping_method->get_meta( 'destination_branch' );
			if ( empty( $selected_branch ) ) {
				Helper::add_error( __( 'Tried to process a shipment to branch without having one destination branch selected', 'wc-oca' ) );
				return;
			}
			$to->set_shipping_branch( $selected_branch['id'] );

			$shipping_method->update_meta_data( 'destination_branch', $selected_branch );
			$shipping_method->save();
		}

		// Considerations from door
		if ( 'PaP' === $shipping_code['type'] || 'PaS' === $shipping_code['type'] ) {
			$seller->get_address()->set_shipping_branch( '0' );
		}

		// Force price 0
		if ( ! Helper::is_enabled( 'enable_insurance' ) || $items->totals()['price'] > 9999999 ) {
			$items->set_items_price( 0 );
		}

		$items = apply_filters( 'wc_oca_items_before_process', $items, $to, $order );
		$to    = apply_filters( 'wc_oca_to_before_process', $to, $items, $order );
		$this->oca_sdk->set_data( $seller );
		$this->oca_sdk->set_shipping_code( $shipping_code['code'] );

		do_action( 'wc_oca_before_order_process' );

		$res = $this->oca_sdk->process_order( $seller, $to->to_array(), $items->items(), $order, $opts );
		if ( ! empty( $res ) && empty( $res['error'] ) && ! empty( $res['tracking_number'] ) && ! empty( $res['oca_number'] ) ) {

			$res = apply_filters( 'wc_oca_res_after_process', $res, $order );

			$shipping_method->update_meta_data( 'tracking_number', $res['tracking_number'] );
			$shipping_method->update_meta_data( 'oca_number', $res['oca_number'] );
			$shipping_method->update_meta_data( 'oca_canceled', false );
			$shipping_method->save();

			$order->add_order_note(
				sprintf( __( 'Your order has been processed with OCA, tracking number: %s', 'wc-oca' ), $res['tracking_number'] ),
				Helper::is_enabled( 'notify_customer_order_processed' ) ? 1 : 0
			);

			Helper::add_success( sprintf( __( 'Order %1$s processed with OCA succesfully, tracking number: %2$s', 'wc-oca' ), $order->get_id(), $res['tracking_number'] ) );

			$this->all_labels_processor->create_order_labels( $order );

			do_action( 'wc_oca_after_order_process', $shipping_method, $order );

		} else {
			if ( ! empty( $res['error'] ) ) {
				Helper::add_error( $res['error'] );
			} else {
				Helper::add_error( __( 'There was an error processing the order with OCA, please try again', 'wc-oca' ) );
			}
		}
	}
}
